﻿using System;
using System.Collections.Generic;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.Security.Cryptography;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI;
using Windows.UI.Core;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Automation.Peers;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 현재 페이지
        /// </summary>
        public static MainPage CurrentPage;

        /// <summary>
        /// 파일명
        /// </summary>
        public static readonly string FileName = "sample.dat";

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 시나리오 리스트
        /// </summary>
        private List<Scenario> scenarioList = new List<Scenario>
        {
            new Scenario() { Title = "파일 생성"                                   , ClassType = typeof(SamplePage01) },
            new Scenario() { Title = "파일의 부모 폴더 조회"                       , ClassType = typeof(SamplePage02) },
            new Scenario() { Title = "파일에 텍스트 쓰기/읽기"                     , ClassType = typeof(SamplePage03) },
            new Scenario() { Title = "파일에 바이트 쓰기/읽기"                     , ClassType = typeof(SamplePage04) },
            new Scenario() { Title = "스트림을 사용해 쓰기/읽기"                   , ClassType = typeof(SamplePage05) },
            new Scenario() { Title = "파일 속성 표시"                              , ClassType = typeof(SamplePage06) },
            new Scenario() { Title = "향후 사용을 위해 저장소 항목에 지속적 액세스", ClassType = typeof(SamplePage07) },
            new Scenario() { Title = "파일 복사"                                   , ClassType = typeof(SamplePage08) },
            new Scenario() { Title = "동일 파일 확인을 위해 두 파일 비교"          , ClassType = typeof(SamplePage09) },
            new Scenario() { Title = "파일 삭제"                                   , ClassType = typeof(SamplePage10) },
            new Scenario() { Title = "실패시 오류없이 파일 조회 시도"              , ClassType = typeof(SamplePage11) }
        };

        /// <summary>
        /// 샘플 저장소 파일
        /// </summary>
        public StorageFile SampleStorageFile = null;

        /// <summary>
        /// MRU 토큰
        /// </summary>
        public string mruToken = null;

        /// <summary>
        /// FAL 토큰
        /// </summary>
        public string falToken = null;

        /// <summary>
        /// E_NO_UNICODE_TRANSLATION
        /// </summary>
        public const int E_NO_UNICODE_TRANSLATION = unchecked((int)0x80070459);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "StorageFile 클래스 사용하기";

            #endregion

            CurrentPage = this;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Internal

        #region 버퍼 구하기 - GetBuffer(source)

        /// <summary>
        /// 버퍼 구하기
        /// </summary>
        /// <param name="source">소스 문자열</param>
        /// <returns>버퍼</returns>
        internal static IBuffer GetBuffer(string source)
        {
            if(string.IsNullOrEmpty(source))
            {
                return new Windows.Storage.Streams.Buffer(0);
            }
            else
            {
                return CryptographicBuffer.ConvertStringToBinary(source, BinaryStringEncoding.Utf8);
            }
        }

        #endregion
        #region 문자열 구하기 - GetString(buffer)

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <param name="buffer">버퍼</param>
        /// <returns>문자열</returns>
        internal static string GetString(IBuffer buffer)
        {
            // 버퍼가 적절히 인코딩되지 않는 경우 E_NO_UNICODE_TRANSLATION 예외가 발생한다.
            return CryptographicBuffer.ConvertBinaryToString(BinaryStringEncoding.Utf8, buffer);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Internal

        #region 파일 부재 상태 메시지 표시하기 - ShowFileNotFoundStatusMessage()

        /// <summary>
        /// 파일 부재 상태 메시지 표시하기
        /// </summary>
        internal void ShowFileNotFoundStatusMessage()
        {
            ShowStatusMessage
            (
                $"'{FileName}' 파일이 없습니다. 1번 시나리오를 사용해 이 파일을 생성하십시오.",
                NotifyType.ErrorMessage
            );
        }

        #endregion
        #region 파일 검증하기 - ValidateFile()

        /// <summary>
        /// 파일 검증하기
        /// </summary>
        internal async void ValidateFile()
        {
            StorageFolder storageFolder = await KnownFolders.GetFolderForUserAsync(null /* 현재 사용자 */, KnownFolderId.PicturesLibrary);

            SampleStorageFile = (await storageFolder.TryGetItemAsync(FileName)) as StorageFile;

            if(SampleStorageFile == null)
            {
                ShowFileNotFoundStatusMessage();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            List<Scenario> sourceList = new List<Scenario>();

            int i = 1;

            foreach(Scenario scenario in scenarioList)
            {
                sourceList.Add
                (
                    new Scenario
                    {
                        Title     = $"{i++}) {scenario.Title}",
                        ClassType = scenario.ClassType
                    }
                );
            }

            this.listBox.ItemsSource = sourceList;

            if(Window.Current.Bounds.Width < 640)
            {
                this.listBox.SelectedIndex = -1;
            }
            else
            {
                this.listBox.SelectedIndex = 0;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 창 토글 버튼 클릭시 처리하기 - paneToggleButton_Click(sender, e)

        /// <summary>
        /// 창 토글 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void paneToggleButton_Click(object sender, RoutedEventArgs e)
        {
            this.splitView.IsPaneOpen = !splitView.IsPaneOpen;
        }

        #endregion
        #region 리스트 박스 선택 변경시 처리하기 - listBox_SelectionChanged(sender, e)

        /// <summary>
        /// 리스트 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ShowStatusMessage(string.Empty, NotifyType.StatusMessage);

            ListBox listBox = sender as ListBox;

            Scenario scenario = listBox.SelectedItem as Scenario;

            if(scenario != null)
            {
                this.frame.Navigate(scenario.ClassType);

                if(Window.Current.Bounds.Width < 640)
                {
                    this.splitView.IsPaneOpen = false;
                }
            }
        }

        #endregion
        #region 상태 메시지 업데이트하기 - UpdateStatusMessage(message, notifyType)

        /// <summary>
        /// 상태 메시지 업데이트하기
        /// </summary>
        /// <param name="message">메시지</param>
        /// <param name="notifyType">통지 타입</param>
        private void UpdateStatusMessage(string message, NotifyType notifyType)
        {
            switch(notifyType)
            {
                case NotifyType.StatusMessage :

                    this.statusBorder.Background = new SolidColorBrush(Colors.Green);

                    break;

                case NotifyType.ErrorMessage :

                    this.statusBorder.Background = new SolidColorBrush(Colors.Red);

                    break;
            }

            this.statusTextBlock.Text = message;

            this.statusBorder.Visibility = (this.statusTextBlock.Text != string.Empty) ? Visibility.Visible : Visibility.Collapsed;

            if(this.statusTextBlock.Text != string.Empty)
            {
                this.statusBorder.Visibility     = Visibility.Visible;
                this.statusStackPanel.Visibility = Visibility.Visible;
            }
            else
            {
                this.statusBorder.Visibility     = Visibility.Collapsed;
                this.statusStackPanel.Visibility = Visibility.Collapsed;
            }

            AutomationPeer peer = FrameworkElementAutomationPeer.FromElement(this.statusTextBlock);

            if(peer != null)
            {
                peer.RaiseAutomationEvent(AutomationEvents.LiveRegionChanged);
            }
        }

        #endregion
        #region 상태 메시지 표시하기 - ShowStatusMessage(message, notifyType)

        /// <summary>
        /// 상태 메시지 표시하기
        /// </summary>
        /// <param name="message">메시지</param>
        /// <param name="notifyType">통지 타입</param>
        public async void ShowStatusMessage(string message, NotifyType notifyType)
        {
            if(Dispatcher.HasThreadAccess)
            {
                UpdateStatusMessage(message, notifyType);
            }
            else
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => UpdateStatusMessage(message, notifyType));
            }
        }

        #endregion
    }
}