﻿using System;
using System.IO;
using Windows.Storage;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// Writing and reading text in a file.
    /// </summary>
    public sealed partial class SamplePage03 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 페이지
        /// </summary>
        private MainPage mainPage = MainPage.CurrentPage;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage03()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage03()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.mainPage.ValidateFile();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 텍스트 쓰기 버튼 클릭시 처리하기 - writeTextButton_Click(sender, e)

        /// <summary>
        /// 텍스트 쓰기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void writeTextButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                try
                {
                    string fileContent = this.sourceTextBox.Text;

                    await FileIO.WriteTextAsync(storageFile, fileContent);

                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에 다음과 같이 텍스트를 썼습니다 :\n{fileContent}",
                        NotifyType.StatusMessage
                    );
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에 텍스트를 쓰는 중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
        #region 텍스트 읽기 버튼 클릭시 처리하기 - readTextButton_Click(sender, e)

        /// <summary>
        /// 텍스트 읽기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void readTextButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                try
                {
                    string fileContent = await FileIO.ReadTextAsync(storageFile);

                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에서 다음과 같이 텍스트를 읽었습니다 :\n{fileContent}",
                        NotifyType.StatusMessage
                    );
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception) when (exception.HResult == MainPage.E_NO_UNICODE_TRANSLATION)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        "파일을 유니코드로 디코딩할 수 없습니다.",
                        NotifyType.ErrorMessage
                    );
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에서 읽는 중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
    }
}