﻿using System;
using System.IO;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 샘플 페이지 4
    /// </summary>
    public sealed partial class SamplePage04 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 페이지
        /// </summary>
        private MainPage mainPage = MainPage.CurrentPage;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage04()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage04()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.mainPage.ValidateFile();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 바이트 쓰기 버튼 클릭시 처리하기 - writeByteButton_Click(sender, e)

        /// <summary>
        /// 바이트 쓰기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void writeByteButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                try
                {
                    string fileContent = this.sourceTextBox.Text;

                    IBuffer buffer = MainPage.GetBuffer(fileContent);

                    await FileIO.WriteBufferAsync(storageFile, buffer);

                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에 다음과 같이 {buffer.Length} 바이트의 텍스트를 썼습니다 :{Environment.NewLine}{fileContent}",
                        NotifyType.StatusMessage
                    );
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에 쓰는 중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
        #region 바이트 읽기 버튼 클릭시 처리하기 - readByteButton_Click(sender, e)

        /// <summary>
        /// 바이트 읽기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void readByteButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                try
                {
                    IBuffer buffer = await FileIO.ReadBufferAsync(storageFile);

                    string fileContent = MainPage.GetString(buffer);

                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에서 다음과 같이 {buffer.Length} 바이트의 텍스트를 읽었습니다 :\n{fileContent}",
                        NotifyType.StatusMessage
                    );
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception) when (exception.HResult == MainPage.E_NO_UNICODE_TRANSLATION)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        "파일이 UTF-8로 인코딩되어 있지 않습니다.",
                        NotifyType.ErrorMessage
                    );
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에서 읽는 중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
    }
}