﻿using System;
using System.IO;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 샘플 페이지 5
    /// </summary>
    public sealed partial class SamplePage05 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 페이지
        /// </summary>
        private MainPage mainPage = MainPage.CurrentPage;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage05()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage05()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.mainPage.ValidateFile();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 스트림 쓰기 버튼 클릭시 처리하기 - writeStreamButton_Click(sender, e)

        /// <summary>
        /// 스트림 쓰기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void writeStreamButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                try
                {
                    string fileContent = this.sourceTextBox.Text;

                    using(StorageStreamTransaction transaction = await storageFile.OpenTransactedWriteAsync())
                    {
                        IBuffer buffer = MainPage.GetBuffer(fileContent);

                        await transaction.Stream.WriteAsync(buffer);

                        transaction.Stream.Size = buffer.Length;

                        await transaction.CommitAsync();

                        this.mainPage.ShowStatusMessage
                        (
                            $"'{storageFile.Name}' 파일에 스트림을 사용해 다음과 같이 텍스트를 썼습니다 :\n{fileContent}",
                            NotifyType.StatusMessage
                        );
                    }
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에 쓰는 중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
        #region 스트림 읽기 버튼 클릭시 처리하기 - readStreamButton_Click(sender, e)

        /// <summary>
        /// 스트림 읽기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void readStreamButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                try
                {
                    using(IRandomAccessStream stream = await storageFile.OpenAsync(FileAccessMode.Read))
                    {
                        ulong size64 = stream.Size;

                        if(size64 <= uint.MaxValue)
                        {
                            uint size32 = (uint)size64;

                            IBuffer buffer = new Windows.Storage.Streams.Buffer(size32);

                            buffer = await stream.ReadAsync(buffer, size32, InputStreamOptions.None);

                            string fileContent = MainPage.GetString(buffer);

                            this.mainPage.ShowStatusMessage
                            (
                                $"'{storageFile.Name}' 파일에서 스트림을 사용해 다음과 같이 텍스트를 읽었습니다 :\n{fileContent}",
                                NotifyType.StatusMessage
                            );
                        }
                        else
                        {
                            this.mainPage.ShowStatusMessage
                            (
                                $"{storageFile.Name} 파일이 너무 커서 ReadAsync가 단일 청크에서 읽을 수 없습니다. " +
                                 "4GB보다 큰 파일은 ReadAsync에서 읽을 수 있도록 여러 청크로 나누어야 합니다.",
                                NotifyType.ErrorMessage
                            );
                        }
                    }
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception) when (exception.HResult == MainPage.E_NO_UNICODE_TRANSLATION)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        "파일이 UTF-8로 인코딩되어 있지 않습니다.",
                        NotifyType.ErrorMessage
                    );
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일에서 읽는 중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
    }
}