﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Windows.Storage;
using Windows.Storage.FileProperties;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 샘플 페이지 6
    /// </summary>
    public sealed partial class SamplePage06 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 페이지
        /// </summary>
        private MainPage mainPage = MainPage.CurrentPage;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage06()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage06()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.mainPage.ValidateFile();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 속성 표시 버튼 클릭시 처리하기 - showPropertyButton_Click(sender, e)

        /// <summary>
        /// 속성 표시 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void showPropertyButton_Click(object sender, RoutedEventArgs e)
        {
            const string dateAccessedProperty = "System.DateAccessed";
            const string fileOwnerProperty    = "System.FileOwner";

            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                try
                {
                    StringBuilder stringBuilder = new StringBuilder();

                    stringBuilder.AppendLine($"파일명 : {storageFile.Name}");
                    stringBuilder.AppendLine($"파일 타입 : {storageFile.FileType}");

                    BasicProperties basicProperties = await storageFile.GetBasicPropertiesAsync();

                    stringBuilder.AppendLine($"파일 크기 : {basicProperties.Size} 바이트");
                    stringBuilder.AppendLine($"데이터 수정 : {basicProperties.DateModified}");

                    IDictionary<string, object> dictionary = await storageFile.Properties.RetrievePropertiesAsync
                    (
                        new[]
                        {
                            dateAccessedProperty,
                            fileOwnerProperty
                        }
                    );

                    object propertyValue = dictionary[dateAccessedProperty];

                    if(propertyValue != null)
                    {
                        stringBuilder.AppendLine($"데이터 액세스 : {propertyValue}");
                    }

                    propertyValue = dictionary[fileOwnerProperty];

                    if(propertyValue != null)
                    {
                        stringBuilder.Append($"파일 소유자 : {propertyValue}");
                    }

                    this.mainPage.ShowStatusMessage(stringBuilder.ToString(), NotifyType.StatusMessage);
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일의 속성 조회시 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
    }
}