﻿using System;
using System.Text;
using Windows.Storage;
using Windows.Storage.AccessCache;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 샘플 페이지 7
    /// </summary>
    public sealed partial class SamplePage07 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 페이지
        /// </summary>
        private MainPage mainPage = MainPage.CurrentPage;

        /// <summary>
        /// FA_E_MAX_PERSISTED_ITEMS_REACHED
        /// </summary>
        private const int FA_E_MAX_PERSISTED_ITEMS_REACHED = unchecked((int)0x80270220);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage07()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage07()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.mainPage.ValidateFile();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 목록에 추가 버튼 클릭시 처리하기 - addToListButton_Click(sender, e)

        /// <summary>
        /// 목록에 추가 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addToListButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = this.mainPage.SampleStorageFile;

            if(storageFile != null)
            {
                if(this.mruRadioButton.IsChecked.Value)
                {
                    RecentStorageItemVisibility visibility = this.systemMRUCheckBox.IsChecked.Value ? RecentStorageItemVisibility.AppAndSystem :
                                                                                                      RecentStorageItemVisibility.AppOnly;

                    this.mainPage.mruToken = StorageApplicationPermissions.MostRecentlyUsedList.Add(storageFile, storageFile.Name, visibility);

                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일이 MRU 목록에 추가되었고 토큰이 저장되었습니다.",
                        NotifyType.StatusMessage
                    );
                }
                else
                {
                    try
                    {
                        this.mainPage.falToken = StorageApplicationPermissions.FutureAccessList.Add(storageFile, storageFile.Name);

                        this.mainPage.ShowStatusMessage
                        (
                            $"'{storageFile.Name}' 파일이 FAL 목록에 추가되었고 토큰이 저장되었습니다.",
                            NotifyType.StatusMessage
                        );
                    }
                    catch(Exception exception) when (exception.HResult == FA_E_MAX_PERSISTED_ITEMS_REACHED)
                    {
                        this.mainPage.ShowStatusMessage
                        (
                            $"FAL 목록이 가득차서 '{storageFile.Name}' 파일이 FAL 목록에 추가되지 않았습니다.",
                            NotifyType.ErrorMessage
                        );
                    }
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
        #region 목록 표시 버튼 클릭시 처리하기 - showListButton_Click(sender, e)

        /// <summary>
        /// 목록 표시 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void showListButton_Click(object sender, RoutedEventArgs e)
        {
            AccessListEntryView entryView;

            string listName;

            if(this.mruRadioButton.IsChecked.Value)
            {
                listName = "MRU";

                entryView = StorageApplicationPermissions.MostRecentlyUsedList.Entries;
            }
            else
            {
                listName = "FAL";

                entryView = StorageApplicationPermissions.FutureAccessList.Entries;
            }

            if(entryView.Count > 0)
            {
                StringBuilder stringBuilder = new StringBuilder($"{listName} 목록은 다음 항목을 포함합니다 : ");

                foreach(AccessListEntry entry in entryView)
                {
                    stringBuilder.AppendLine();

                    stringBuilder.Append(entry.Metadata);
                }

                this.mainPage.ShowStatusMessage(stringBuilder.ToString(), NotifyType.StatusMessage);
            }
            else
            {
                this.mainPage.ShowStatusMessage
                (
                    $"{listName} 목록이 비어 있습니다.",
                    NotifyType.ErrorMessage
                );
            }
        }

        #endregion
        #region 목록에서 열기 버튼 클릭시 처리하기 - openFromListButton_Click(sender, e)

        /// <summary>
        /// 목록에서 열기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void openFromListButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile storageFile = null;

            if(this.mruRadioButton.IsChecked.Value)
            {
                if(this.mainPage.mruToken != null)
                {
                    if(StorageApplicationPermissions.MostRecentlyUsedList.ContainsItem(this.mainPage.mruToken))
                    {
                        storageFile = await StorageApplicationPermissions.MostRecentlyUsedList.GetFileAsync(this.mainPage.mruToken);
                    }
                    else
                    {
                        this.mainPage.ShowStatusMessage
                        (
                            "토큰이 더 이상 유효하지 않습니다.",
                            NotifyType.ErrorMessage
                        );
                    }
                }
                else
                {
                    this.mainPage.ShowStatusMessage
                    (
                        "이 작업은 토큰을 요구합니다. 먼저 MRU 목록에 파일을 추가합니다.",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                if(this.mainPage.falToken != null)
                {
                    storageFile = await StorageApplicationPermissions.FutureAccessList.GetFileAsync(this.mainPage.falToken);
                }
                else
                {
                    this.mainPage.ShowStatusMessage
                    (
                        "이 작업은 토큰을 요구합니다. 먼저 FAL 목록에 파일을 추가합니다.",
                        NotifyType.ErrorMessage
                    );
                }
            }

            if(storageFile != null)
            {
                try
                {
                    string fileContent = await FileIO.ReadTextAsync(storageFile);
 
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{storageFile.Name}' 파일이 저장된 토큰에 의해 열렸습니다. 다음 텍스트를 포함합니다 :\n{fileContent}",
                        NotifyType.StatusMessage
                    );

                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"목록에서 열려진 파일을 읽는 중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
        }

        #endregion
    }
}