﻿using System;
using System.IO;
using Windows.Storage;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 샘플 페이지 8
    /// </summary>
    public sealed partial class SamplePage08 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 메인 페이지
        /// </summary>
        private MainPage mainPage = MainPage.CurrentPage;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage08()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage08()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 발생자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            this.mainPage.ValidateFile();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 파일 복사 버튼 클릭시 처리하기 - copyFileButton_Click(sender, e)

        /// <summary>
        /// 파일 복사 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void copyFileButton_Click(object sender, RoutedEventArgs e)
        {
            StorageFile sourceStorageFile = this.mainPage.SampleStorageFile;

            if(sourceStorageFile != null)
            {
                try
                {
                    StorageFolder stroageFolder = await KnownFolders.GetFolderForUserAsync(null /* 현재 사용자 */, KnownFolderId.PicturesLibrary);

                    StorageFile targetStorageFile = await sourceStorageFile.CopyAsync
                    (
                        stroageFolder,
                        "sample-copy.dat",
                        NameCollisionOption.ReplaceExisting
                    );

                    this.mainPage.ShowStatusMessage
                    (
                        $"'{sourceStorageFile.Name}' 파일이 복사되었고 새 파일명은 '{targetStorageFile.Name}' 입니다.",
                        NotifyType.StatusMessage
                    );
                }
                catch(FileNotFoundException)
                {
                    this.mainPage.ShowFileNotFoundStatusMessage();
                }
                catch(Exception exception)
                {
                    this.mainPage.ShowStatusMessage
                    (
                        $"'{sourceStorageFile.Name}' 파일 복사중 에러가 발생했습니다 : {exception.Message}",
                        NotifyType.ErrorMessage
                    );
                }
            }
            else
            {
                this.mainPage.ShowFileNotFoundStatusMessage();
            }
        }

        #endregion
    }
}